Classifier Module
================
### Overview

The classifier module provides a base class `BaseClassifier` and a concrete implementation `SimpleMLP` for classification tasks. The module uses PyTorch for building and training neural networks.

### Classes

#### BaseClassifier
##### Description
The `BaseClassifier` class serves as the base class for all classifiers. It defines the interface for identification, verification, and authentication tasks.

##### Methods
* `identification(X_test)`: Abstract method for identification tasks.
* `verification(X_test)`: Abstract method for verification tasks.
* `authentication(X_auth)`: Abstract method for authentication tasks.

#### SimpleMLP
##### Description
The `SimpleMLP` class is a concrete implementation of the `BaseClassifier` class. It uses a simple multi-layer perceptron (MLP) neural network for classification tasks.

##### Attributes
* `device`: The device used for training and evaluation (e.g., CUDA or CPU).
* `model`: The PyTorch neural network model.
* `optimizer`: The optimizer used for training the model.
* `loss`: The loss function used for training the model.

##### Methods
* `__init__(input_nodes, output_nodes, middle_layer_size=1024)`: Initializes the `SimpleMLP` classifier.
* `__str__()`: Returns a string representation of the classifier.
* `training(train_data)`: Trains the classifier using the provided `train_data`.
* `identification(X_id)`: Evaluates the classifier for identification tasks.
* `verification(X_ver)`: Evaluates the classifier for verification tasks.
* `authentication(X_auth)`: Evaluates the classifier for authentication tasks.

#### MLPModel
##### Description
The `MLPModel` class is a PyTorch neural network model used by the `SimpleMLP` classifier.

##### Attributes
* `linear1`: The first linear layer of the neural network.
* `activation`: The activation function used by the neural network.
* `dropout`: The dropout layer used by the neural network.
* `linear2`: The second linear layer of the neural network.
* `softmax`: The softmax layer used by the neural network.

##### Methods
* `forward(x)`: Defines the forward pass of the neural network.

### Functions

None

### Example Usage
```python
# Load the required data
config = {"user_num": 15, "window": 2, "window_step": 0.1}
ret = MedbaRetriever(config)
ret.load_datasets()

# Train and evaluate identification
id_train, _, id_test, _ = ret.retrieve_identification()
identifier = SimpleMLP(
    input_nodes=28, output_nodes=config["user_num"], middle_layer_size=1024
)
identifier.training(id_train)
result = evaluate_identification(identifier, id_test)
result.metrics()
result.visualise()

# Train, evaluate, and compare verification
results = []
for i in range(config["user_num"]):
    ver_train, _, ver_test, ver_adver = ret.retrieve_verification(i)
    verifier = SimpleMLP(input_nodes=28, output_nodes=2, middle_layer_size=1024)
    result = evaluate_verification(verifier, ver_adver)
    results.append(result)
compare_results(results)

# Use the identification model to also evaluate authentication
_, _, _, auth_adver = ret.retrieve_authentication()
result = evaluate_authentication(identifier, auth_adver)
result.metrics()
result.visualise()
```

### Data Retrieval
The classifier module uses data retrievers from the `src.ebat.data.retrievers` module to load the required data. The available data retrievers are:

* `MedbaRetriever`
* `HmogRetriever`
* `UciharRetriever`

Each data retriever has its own configuration options and methods for loading the data. For example:
```python
config = {"user_num": 15, "window": 2, "window_step": 0.1}
ret = MedbaRetriever(config)
ret.load_datasets()
```
[Warning: Text generated by GPT@JRC using Generative AI technology - Please assess this text critically before using it]